@@ -1,15 +1,23 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# -*- coding: utf-8 -*-
+# Copyright 2014-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
-# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Martin Barisits, <martin.barisits@cern.ch>, 2014
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
-# - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
-#
-# PY3K COMPATIBLE
+# - Martin Barisits <martin.barisits@cern.ch>, 2014-2021
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 
 import logging
 
@@ -36,8 +44,8 @@ def get_dataset_locks(scope, name, vo='def'):
 
     locks = lock.get_dataset_locks(scope=scope, name=name)
 
-    for l in locks:
-        yield api_update_return_dict(l)
+    for lock_object in locks:
+        yield api_update_return_dict(lock_object)
 
 
 def get_dataset_locks_by_rse(rse, vo='def'):
@@ -52,8 +60,8 @@ def get_dataset_locks_by_rse(rse, vo='def'):
     rse_id = get_rse_id(rse=rse, vo=vo)
     locks = lock.get_dataset_locks_by_rse_id(rse_id=rse_id)
 
-    for l in locks:
-        yield api_update_return_dict(l)
+    for lock_object in locks:
+        yield api_update_return_dict(lock_object)
 
 
 def get_replica_locks_for_rule_id(rule_id, vo='def'):
@@ -67,8 +75,8 @@ def get_replica_locks_for_rule_id(rule_id, vo='def'):
 
     locks = lock.get_replica_locks_for_rule_id(rule_id=rule_id)
 
-    for l in locks:
-        if l['scope'].vo != vo:  # rule is on a different VO, so don't return any locks
+    for lock_object in locks:
+        if lock_object['scope'].vo != vo:  # rule is on a different VO, so don't return any locks
             LOGGER.debug('rule id %s is not present on VO %s' % (rule_id, vo))
             break
-        yield api_update_return_dict(l)
+        yield api_update_return_dict(lock_object)
