@@ -20,7 +20,7 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2021
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2013
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2016-2021
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
 # - Tobias Wegner <twegner@cern.ch>, 2018-2019
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
@@ -31,93 +31,52 @@
 # - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
 # - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - root <root@escape-rucio-dev-oidc-r.cern.ch>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Anil Panta <anil.panta@desy.de>, 2021
+# - Mayank Sharma <mayank.sharma@cern.ch>, 2021
+# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Anil Panta <47672624+panta-123@users.noreply.github.com>, 2021
+# - Ilija Vukotic <ivukotic@cern.ch>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 from __future__ import absolute_import, print_function
 
-try:
-    import importlib
-    importlib.util.find_spec('')
-except AttributeError:
-    pass
-
-# Fallback until Python <3.9
-# Must be removed with Python 3.9
-try:
-    import imp
-except AttributeError:
-    pass
-
 import base64
-import copy
 import datetime
 import errno
 import getpass
 import hashlib
+import io
 import json
 import logging
+import mmap
 import os
 import os.path
 import re
 import socket
 import subprocess
-import sys
 import tempfile
 import threading
 import time
 import zlib
 from enum import Enum
+from functools import partial
 from uuid import uuid4 as uuid
 from xml.etree import ElementTree
 
 import requests
-from six import string_types, text_type, PY3
+from six import string_types, text_type, binary_type, ensure_text, PY3
+from six.moves import StringIO, zip_longest as izip_longest
+from six.moves.urllib.parse import urlparse, urlencode, quote, parse_qsl, urlunparse
+from six.moves.configparser import NoOptionError, NoSectionError
 
 from rucio.common.config import config_get
 from rucio.common.exception import MissingModuleException, InvalidType, InputValidationError, MetalinkJsonParsingError, RucioException
+from rucio.common.extra import import_extras
 from rucio.common.types import InternalAccount, InternalScope
 
-try:
-    # Python 2
-    from itertools import izip_longest
-except ImportError:
-    # Python 3
-    from itertools import zip_longest as izip_longest
-try:
-    # Python 2
-    from urllib import urlencode, quote
-except ImportError:
-    # Python 3
-    from urllib.parse import urlencode, quote
-try:
-    # Python 2
-    from StringIO import StringIO
-except ImportError:
-    # Python 3
-    from io import StringIO
-try:
-    # Python 2
-    import urlparse
-except ImportError:
-    # Python 3
-    import urllib.parse as urlparse
-
-# Extra modules: Only imported if available
-EXTRA_MODULES = {'paramiko': False}
-
-for extra_module in EXTRA_MODULES:
-    if 'imp' in sys.modules:
-        try:
-            imp.find_module(extra_module)
-            EXTRA_MODULES[extra_module] = True
-        except ImportError:
-            EXTRA_MODULES[extra_module] = False
-    else:
-        if importlib.util.find_spec(extra_module):
-            EXTRA_MODULES[extra_module] = True
-        else:
-            EXTRA_MODULES[extra_module] = False
+EXTRA_MODULES = import_extras(['paramiko'])
 
 if EXTRA_MODULES['paramiko']:
     try:
@@ -174,28 +133,6 @@ def build_url(url, path=None, params=None, doseq=False):
     return complete_url
 
 
-def oidc_identity_string(sub, iss):
-    """
-    Transform IdP sub claim and issuers url into users identity string.
-    :param sub: users SUB claim from the Identity Provider
-    :param iss: issuer (IdP) https url
-
-    :returns: OIDC identity string "SUB=<usersid>, ISS=https://iam-test.ch/"
-    """
-    return 'SUB=' + str(sub) + ', ISS=' + str(iss)
-
-
-def sqlalchemy_obj_to_dict(sqlalchemyresult):
-    """
-    Makes dictionary from SQLAlchemy query result object
-    :param sqlalchemyresult:
-    :returns: dictionary
-    """
-    res_dict = copy.deepcopy(dict(sqlalchemyresult.__dict__))
-    del res_dict['_sa_instance_state']
-    return res_dict
-
-
 def all_oidc_req_claims_present(scope, audience, required_scope, required_audience, sepatator=" "):
     """
     Checks if both of the following statements are true:
@@ -219,8 +156,7 @@ def all_oidc_req_claims_present(scope, audience, required_scope, required_audien
         required_scope = ""
     if not required_audience:
         required_audience = ""
-    if (isinstance(scope, list) and isinstance(audience, list)  # NOQA: W504
-        and isinstance(required_scope, list) and isinstance(required_audience, list)):
+    if (isinstance(scope, list) and isinstance(audience, list) and isinstance(required_scope, list) and isinstance(required_audience, list)):
         scope = [str(it) for it in scope]
         audience = [str(it) for it in audience]
         required_scope = [str(it) for it in required_scope]
@@ -306,13 +242,16 @@ def adler32(file):
     adler = 1
 
     try:
-        with open(file, 'rb') as openFile:
-            for line in openFile:
-                adler = zlib.adler32(line, adler)
+        with open(file, 'r+b') as f:
+            # memory map the file
+            m = mmap.mmap(f.fileno(), 0)
+            # partial block reads at slightly increased buffer sizes
+            for block in iter(partial(m.read, io.DEFAULT_BUFFER_SIZE), b''):
+                adler = zlib.adler32(block, adler)
     except Exception as e:
         raise Exception('FATAL - could not get Adler32 checksum of file %s - %s' % (file, e))
 
-    # backflip on 32bit
+    # backflip on 32bit -- can be removed once everything is fully migrated to 64bit
     if adler < 0:
         adler = adler + 2 ** 32
 
@@ -434,10 +373,10 @@ def render_json(**data):
     return json.dumps(data, cls=APIEncoder)
 
 
-def render_json_list(l):
+def render_json_list(list):
     """ JSON render function for list
     """
-    return json.dumps(l, cls=APIEncoder)
+    return json.dumps(list, cls=APIEncoder)
 
 
 def datetime_parser(dct):
@@ -503,12 +442,12 @@ def grouper(iterable, n, fillvalue=None):
     return izip_longest(*args, fillvalue=fillvalue)
 
 
-def chunks(l, n):
+def chunks(list, n):
     """
     Yield successive n-sized chunks from l.
     """
-    for i in range(0, len(l), n):
-        yield l[i:i + n]
+    for i in range(0, len(list), n):
+        yield list[i:i + n]
 
 
 def my_key_generator(namespace, fn, **kw):
@@ -601,6 +540,7 @@ def construct_surl_BelleII(dsn, filename):
 
 _SURL_ALGORITHMS = {}
 _DEFAULT_SURL = 'DQ2'
+_loaded_policy_modules = False
 
 
 def register_surl_algorithm(surl_callable, name=None):
@@ -614,9 +554,39 @@ register_surl_algorithm(construct_surl_DQ2, 'DQ2')
 register_surl_algorithm(construct_surl_BelleII, 'BelleII')
 
 
+def _register_policy_package_surl_algorithms():
+    def try_importing_policy(vo=None):
+        import importlib
+        try:
+            package = config.config_get('policy', 'package' + ('' if not vo else '-' + vo['vo']))
+            module = importlib.import_module(package)
+            if hasattr(module, 'get_surl_algorithms'):
+                _SURL_ALGORITHMS.update(module.get_surl_algorithms())
+        except (NoOptionError, NoSectionError, ImportError):
+            pass
+
+    from rucio.common import config
+    from rucio.core.vo import list_vos
+    try:
+        multivo = config.config_get_bool('common', 'multi_vo')
+    except (NoOptionError, NoSectionError):
+        multivo = False
+    if not multivo:
+        # single policy package
+        try_importing_policy()
+    else:
+        # policy package per VO
+        vos = list_vos()
+        for vo in vos:
+            try_importing_policy(vo)
+
+
 def construct_surl(dsn, filename, naming_convention=None):
-    # ensure that policy package is loaded in case it registers its own algorithms
-    import rucio.common.schema  # noqa: F401
+    global _loaded_policy_modules
+    if not _loaded_policy_modules:
+        # on first call, register any SURL functions from the policy packages
+        _register_policy_package_surl_algorithms()
+        _loaded_policy_modules = True
 
     if naming_convention is None or naming_convention not in _SURL_ALGORITHMS:
         naming_convention = _DEFAULT_SURL
@@ -893,39 +863,58 @@ class Color:
 
 def detect_client_location():
     """
-    Open a UDP socket to a machine on the internet, to get the local IPv4 and IPv6
-    addresses of the requesting client.
-
+    Normally client IP will be set on the server side (request.remote_addr)
+    Here setting ip on the one seen by the host itself. There is no connection
+    to Google DNS servers.
     Try to determine the sitename automatically from common environment variables,
     in this order: SITE_NAME, ATLAS_SITE_NAME, OSG_SITE_NAME. If none of these exist
     use the fixed string 'ROAMING'.
+
+    If environment variables sets location, it uses it.
     """
 
-    ip = '0.0.0.0'
+    ip = None
+
     try:
-        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        s.connect(("8.8.8.8", 80))
+        s = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
+        s.connect(("2001:4860:4860:0:0:0:0:8888", 80))
         ip = s.getsockname()[0]
     except Exception:
         pass
 
-    ip6 = '::'
-    try:
-        s = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
-        s.connect(("2001:4860:4860:0:0:0:0:8888", 80))
-        ip6 = s.getsockname()[0]
-    except Exception:
-        pass
+    if not ip:
+        try:
+            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+            s.connect(("8.8.8.8", 80))
+            ip = s.getsockname()[0]
+        except Exception:
+            pass
+
+    if not ip:
+        ip = '0.0.0.0'
 
     site = os.environ.get('SITE_NAME',
                           os.environ.get('ATLAS_SITE_NAME',
                                          os.environ.get('OSG_SITE_NAME',
                                                         'ROAMING')))
 
+    latitude = os.environ.get('RUCIO_LATITUDE')
+    longitude = os.environ.get('RUCIO_LONGITUDE')
+    if latitude and longitude:
+        try:
+            latitude = float(latitude)
+            longitude = float(longitude)
+        except ValueError:
+            latitude = longitude = 0
+            print('Client set latitude and longitude are not valid.')
+    else:
+        latitude = longitude = None
+
     return {'ip': ip,
-            'ip6': ip6,
             'fqdn': socket.getfqdn(),
-            'site': site}
+            'site': site,
+            'latitude': latitude,
+            'longitude': longitude}
 
 
 def ssh_sign(private_key, message):
@@ -1001,11 +990,11 @@ def add_url_query(url, query):
     :return: The expanded URL with the new query parameters
     """
 
-    url_parts = list(urlparse.urlparse(url))
-    mod_query = dict(urlparse.parse_qsl(url_parts[4]))
+    url_parts = list(urlparse(url))
+    mod_query = dict(parse_qsl(url_parts[4]))
     mod_query.update(query)
     url_parts[4] = urlencode(mod_query)
-    return urlparse.urlunparse(url_parts)
+    return urlunparse(url_parts)
 
 
 def get_bytes_value_from_string(input_string):
@@ -1238,6 +1227,9 @@ def run_cmd_process(cmd, timeout=3600):
         process.kill()
 
     stdout, stderr = process.communicate()
+    if isinstance(stdout, binary_type):
+        stdout = ensure_text(stdout, errors='replace')
+        stderr = ensure_text(stderr, errors='replace')
     if not stderr:
         stderr = ''
     if not stdout:
@@ -1312,34 +1304,6 @@ def get_parsed_throttler_mode(throttler_mode):
     return (direction, all_activities)
 
 
-def query_bunches(query, bunch_by):
-    """
-    Queries output by yield_per sqlalchemy function
-    (which in a for loop returns rows one by one).
-    Groups the query rows in bunches of bunch_by
-    elements and returns list of bunches.
-    :param query: sqlalchemy session query
-    :param bunch_by: integer number
-    :returns: [[bunch_of_tuples_1],[bunch_of_tuples_2],...]
-
-    """
-    filtered_bunches = []
-    item_bunch = []
-    for i in query.yield_per(bunch_by):
-        # i is either tuple of one element (token/model object etc.)
-        if not isinstance(i, tuple) and not isinstance(i, list):
-            item_bunch.append(i)
-        # or i is a tuple with the column elements per row
-        else:
-            item_bunch += i
-        if len(item_bunch) % bunch_by == 0:
-            filtered_bunches.append(item_bunch)
-            item_bunch = []
-    if item_bunch:
-        filtered_bunches.append(item_bunch)
-    return filtered_bunches
-
-
 def setup_logger(module_name=None, logger_name=None, logger_level=None, verbose=False):
     '''
     Factory method to set logger with handlers.
@@ -1418,6 +1382,15 @@ def setup_logger(module_name=None, logger_name=None, logger_level=None, verbose=
     return logger
 
 
+def daemon_sleep(start_time, sleep_time, graceful_stop, logger=logging.log):
+    """Sleeps a daemon the time provided by sleep_time"""
+    end_time = time.time()
+    time_diff = end_time - start_time
+    if time_diff < sleep_time:
+        logger(logging.INFO, 'Sleeping for a while :  %s seconds', (sleep_time - time_diff))
+        graceful_stop.wait(sleep_time - time_diff)
+
+
 class retry:
     """Retry callable object with configuragle number of attempts"""
 
