@@ -1,6 +1,28 @@
+# -*- coding: utf-8 -*-
+# Copyright 2020 CERN
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# Authors:
+# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2020
+# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
+
 from abc import ABCMeta, abstractmethod
+
 from six import add_metaclass
 
+from rucio.db.sqla.session import transactional_session
+
 
 @add_metaclass(ABCMeta)
 class DidMetaPlugin(object):
@@ -40,6 +62,21 @@ class DidMetaPlugin(object):
         """
         pass
 
+    @transactional_session
+    def set_metadata_bulk(self, scope, name, meta, recursive=False, session=None):
+        """
+        Add metadata to data identifier in bulk.
+
+        :param scope: The scope name.
+        :param name: The data identifier name.
+        :param meta: all key-values to set.
+        :type meta: dict
+        :param recursive: Option to propagate the metadata change to content.
+        :param session: The database session in use.
+        """
+        for key, value in meta.items():
+            self.set_metadata(scope, name, key, value, recursive=recursive, session=session)
+
     @abstractmethod
     def delete_metadata(self, scope, name, key, session=None):
         """
@@ -70,11 +107,11 @@ class DidMetaPlugin(object):
         pass
 
     @abstractmethod
-    def manages_key(self, key):
+    def manages_key(self, key, session=None):
         """
         Returns whether key is managed by this plugin or not.
-        JSON plugin should be considered a wildcard.
         :param key: Key of the metadata.
+        :param session: The database session in use.
         :returns (Boolean)
         """
         pass
