@@ -1,4 +1,5 @@
-# Copyright 2016-2018 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright 2016-2021 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,18 +14,16 @@
 # limitations under the License.
 #
 # Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2017
-# - Vincent Garonne <vgaronne@gmail.com>, 2016-2018
-# - Tomas Javurek <tomas.javurek@cern.ch>, 2017
+# - Martin Barisits <martin.barisits@cern.ch>, 2016-2021
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2016-2018
+# - Tomas Javurek <tomas.javurek@cern.ch>, 2017-2019
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2017
+# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2020
 # - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
 # - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2019
 # - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin, <patrick.austin@stfc.ac.uk>, 2020
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2021
-#
-# PY3K COMPATIBLE
 
 from __future__ import print_function, division
 
@@ -183,9 +182,9 @@ def _list_rebalance_rule_candidates_dump(rse_id, mode=None):
         return candidates
 
     # looping over the dump and selecting the rules
-    for l in r.iter_lines():
-        if l:
-            file_scope, file_name, rule_id, rse_expression, account, file_size, state = l.split('\t')
+    for line in r.iter_lines():
+        if line:
+            file_scope, file_name, rule_id, rse_expression, account, file_size, state = line.split('\t')
             if rule_id not in rules:
                 rule_info = {}
                 try:
@@ -340,7 +339,7 @@ def select_target_rse(parent_rule, current_rse_id, rse_expression, subscription_
             expression = '((tier=2&type=DATADISK)\\datapolicynucleus=1)\\{}'.format(target_rse)
         rses = parse_expression(expression=expression, filter={'vo': vo, 'availability_write': True}, session=session)
     rseselector = RSESelector(account=InternalAccount('ddmadmin', vo=vo), rses=rses, weight='freespace', copies=1, ignore_account_limit=True, session=session)
-    return get_rse_name([rse_id for rse_id, _, _ in rseselector.select_rse(size=0, preferred_rse_ids=[], blacklist=other_rses)][0], session=session)
+    return get_rse_name([rse_id for rse_id, _, _ in rseselector.select_rse(size=0, preferred_rse_ids=[], blocklist=other_rses)][0], session=session)
 
 
 @transactional_session
