@@ -25,6 +25,7 @@
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Sahan Dilshan <32576163+sahandilshan@users.noreply.github.com>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 """
 Conveyor is a daemon to manage file transfers.
@@ -114,7 +115,7 @@ class Receiver(object):
                             'md5': msg['file_metadata'].get('md5', None),
                             'filesize': msg['file_metadata'].get('filesize', None),
                             'external_host': msg.get('endpnt', None),
-                            'job_m_replica': msg.get('job_m_replica', None),
+                            'multi_sources': msg.get('job_metadata', {}).get('multi_sources', None),
                             'details': {'files': msg['file_metadata']}}
 
                 record_counter('daemons.conveyor.receiver.message_rucio')
@@ -135,7 +136,7 @@ class Receiver(object):
 
                         if self.__full_mode:
                             ret = request.update_request_state(response)
-                            record_counter('daemons.conveyor.receiver.update_request_state.%s' % ret)
+                            record_counter('daemons.conveyor.receiver.update_request_state.{updated}', labels={'updated': ret})
                         else:
                             try:
                                 logging.debug("Update request %s update time" % response['request_id'])
@@ -221,7 +222,7 @@ def receiver(id, total_threads=1, full_mode=False, all_vos=False):
 
             if not conn.is_connected():
                 logging.info('connecting to %s' % conn.transport._Transport__host_and_ports[0][0])
-                record_counter('daemons.messaging.fts3.reconnect.%s' % conn.transport._Transport__host_and_ports[0][0].split('.')[0])
+                record_counter('daemons.messaging.fts3.reconnect.{host}', labels={'host': conn.transport._Transport__host_and_ports[0][0].split('.')[0]})
 
                 conn.set_listener('rucio-messaging-fts3', Receiver(broker=conn.transport._Transport__host_and_ports[0],
                                                                    id=id, total_threads=total_threads,
