@@ -29,6 +29,8 @@
 # - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
 # - Luca Scotto Lavina, <scotto@lpnhe.in2p3.fr>, 2021
 # - Thomas Beermann, <thomas.beermann@cern.ch>, 2021
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2021
+# - Lionel Schwarz <lionel.schwarz@in2p3.fr>, 2021
 #
 # PY3K COMPATIBLE
 
@@ -82,6 +84,7 @@ class RSEDeterministicTranslation(object):
         self.rse = rse
         self.rse_attributes = rse_attributes if rse_attributes else {}
         self.protocol_attributes = protocol_attributes if protocol_attributes else {}
+        self.loaded_policy_modules = False
 
     @classmethod
     def supports(cls, name):
@@ -219,6 +222,27 @@ class RSEDeterministicTranslation(object):
 
         return '%s/%s/%s/%s' % (scope[0:7], scope[4:len(scope)], name.split('-')[0] + "-" + name.split('-')[1], name)
 
+    @staticmethod
+    def __lsst(scope, name, rse, rse_attrs, protocol_attrs):
+        """
+        LFN2PFN algorithm for Rubin-LSST in the ESCAPE project
+
+        Replace convention delimiter '__' by '/'
+        The Escape instance does use the 'generic' Rucio schema.
+
+        :param scope: Scope of the LFN (ignored)
+        :param name: File name of the LFN.
+        :param rse: RSE for PFN (ignored)
+        :param rse_attrs: RSE attributes for PFN (ignored)
+        :param protocol_attrs: RSE protocol attributes for PFN (ignored)
+        :returns: Path for use in the PFN generation.
+        """
+        del scope
+        del rse
+        del rse_attrs
+        del protocol_attrs
+        return name.replace('__', '/')
+
     @classmethod
     def _module_init_(cls):
         """
@@ -229,6 +253,7 @@ class RSEDeterministicTranslation(object):
         cls.register(cls.__ligo, "ligo")
         cls.register(cls.__belleii, "belleii")
         cls.register(cls.__xenon, "xenon")
+        cls.register(cls.__lsst, "lsst")
         policy_module = None
         try:
             policy_module = config.config_get('policy', 'lfn2pfn_module')
@@ -242,6 +267,31 @@ class RSEDeterministicTranslation(object):
 
         cls._DEFAULT_LFN2PFN = config.get_lfn2pfn_algorithm_default()
 
+    def try_importing_policy(self, vo=None):
+        import importlib
+        try:
+            package = config.config_get('policy', 'package' + ('' if not vo else '-' + vo['vo']))
+            module = importlib.import_module(package)
+            if hasattr(module, 'get_lfn2pfn_algorithms'):
+                RSEDeterministicTranslation._LFN2PFN_ALGORITHMS.update(module.get_lfn2pfn_algorithms())
+        except (NoOptionError, NoSectionError, ImportError):
+            pass
+
+    def query_policy_packages(self):
+        from rucio.core.vo import list_vos
+        try:
+            multivo = config.config_get_bool('common', 'multi_vo')
+        except (NoOptionError, NoSectionError):
+            multivo = False
+        if not multivo:
+            # single policy package
+            self.try_importing_policy()
+        else:
+            # policy package per VO
+            vos = list_vos()
+            for vo in vos:
+                self.try_importing_policy(vo)
+
     def path(self, scope, name):
         """ Transforms the logical file name into a PFN's path.
 
@@ -250,8 +300,10 @@ class RSEDeterministicTranslation(object):
 
             :returns: RSE specific URI of the physical file
         """
-        # ensure that policy package is loaded in case it registers algorithms
-        import rucio.common.schema  # noqa: F401
+        # on first call, register any lfn2pfn algorithms from the policy package(s) (server only)
+        if getattr(rsemanager, 'SERVER_MODE', None) and not self.loaded_policy_modules:
+            self.query_policy_packages()
+            self.loaded_policy_modules = True
 
         algorithm = self.rse_attributes.get('lfn2pfn_algorithm', 'default')
         if algorithm == 'default':
@@ -266,10 +318,12 @@ RSEDeterministicTranslation._module_init_()  # pylint: disable=protected-access
 class RSEProtocol(object):
     """ This class is virtual and acts as a base to inherit new protocols from. It further provides some common functionality which applies for the amjority of the protocols."""
 
-    def __init__(self, protocol_attr, rse_settings, logger=None):
+    def __init__(self, protocol_attr, rse_settings, logger=logging.log):
         """ Initializes the object with information about the referred RSE.
 
-            :param props: Properties of the requested protocol
+            :param protocol_attr:  Properties of the requested protocol.
+            :param rse_settting:   The RSE settings.
+            :param logger:         Optional decorated logger that can be passed from the calling daemons or servers.
         """
         self.auth_token = protocol_attr['auth_token']
         protocol_attr.pop('auth_token')
@@ -290,9 +344,6 @@ class RSEProtocol(object):
                 setattr(self, 'lfns2pfns', self.__lfns2pfns_client)
             if getattr(rsemanager, 'SERVER_MODE', None):
                 setattr(self, '_get_path', self._get_path_nondeterministic_server)
-        if not self.logger:
-            self.logger = logging.getLogger('%s.null' % __name__)
-            self.logger.disabled = True
 
     def lfns2pfns(self, lfns):
         """
@@ -323,14 +374,17 @@ class RSEProtocol(object):
                                                          lfn['path'] if not lfn['path'].startswith('/') else lfn['path'][1:]
                                                          ])
             else:
-                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'],
-                                                         '://',
-                                                         self.attributes['hostname'],
-                                                         ':',
-                                                         str(self.attributes['port']),
-                                                         prefix,
-                                                         self._get_path(scope=scope, name=name)
-                                                         ])
+                try:
+                    pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'],
+                                                             '://',
+                                                             self.attributes['hostname'],
+                                                             ':',
+                                                             str(self.attributes['port']),
+                                                             prefix,
+                                                             self._get_path(scope=scope, name=name)
+                                                             ])
+                except exception.ReplicaNotFound as e:
+                    self.logger(logging.WARNING, str(e))
         return pfns
 
     def __lfns2pfns_client(self, lfns):
@@ -396,12 +450,15 @@ class RSEProtocol(object):
             while '//' in parsed.path:
                 parsed = parsed._replace(path=parsed.path.replace('//', '/'))
             path = parsed.path
+            prefix = self.attributes['prefix']
+            while '//' in prefix:
+                prefix = prefix.replace('//', '/')
 
             # Protect against 'lazy' defined prefixes for RSEs in the repository
-            if not self.attributes['prefix'].startswith('/'):
-                self.attributes['prefix'] = '/' + self.attributes['prefix']
-            if not self.attributes['prefix'].endswith('/'):
-                self.attributes['prefix'] += '/'
+            if not prefix.startswith('/'):
+                prefix = '/' + prefix
+            if not prefix.endswith('/'):
+                prefix += '/'
 
             if self.attributes['hostname'] != hostname:
                 if self.attributes['hostname'] != 'localhost':  # In the database empty hostnames are replaced with localhost but for some URIs (e.g. file) a hostname is not included
@@ -410,13 +467,12 @@ class RSEProtocol(object):
             if self.attributes['port'] != port:
                 raise exception.RSEFileNameNotSupported('Invalid port: provided \'%s\', expected \'%s\'' % (port, self.attributes['port']))
 
-            if not path.startswith(self.attributes['prefix']):
-                raise exception.RSEFileNameNotSupported('Invalid prefix: provided \'%s\', expected \'%s\'' % ('/'.join(path.split('/')[0:len(self.attributes['prefix'].split('/')) - 1]),
-                                                                                                              self.attributes['prefix']))  # len(...)-1 due to the leading '/
+            if not path.startswith(prefix):
+                raise exception.RSEFileNameNotSupported('Invalid prefix: provided \'%s\', expected \'%s\'' % ('/'.join(path.split('/')[0:len(prefix.split('/')) - 1]),
+                                                                                                              prefix))  # len(...)-1 due to the leading '/
 
             # Spliting parsed.path into prefix, path, filename
-            prefix = self.attributes['prefix']
-            path = path.partition(self.attributes['prefix'])[2]
+            path = path.partition(prefix)[2]
             name = path.split('/')[-1]
             path = '/'.join(path.split('/')[:-1])
             if not path.startswith('/'):
